/*
 * Moblin-Web-Browser: The web browser for Moblin
 * Copyright (c) 2009, Intel Corporation.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms and conditions of the GNU Lesser General Public License,
 * version 2.1, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.
 */

const MwbIPagesObserver
  = Components.interfaces.MwbIPagesObserver;
const nsISupports = Components.interfaces.nsISupports;
const nsISupportsWeakReference = Components.interfaces.nsISupportsWeakReference;

const N_PINNED_PAGES = 5;

var pagesService;

function FetchFavsObserver()
{
}

FetchFavsObserver.prototype =
{
  favoritesReceived : function(favorites, nFavorites)
  {
    // Remove all existing favorite links
    var favDiv = document.getElementById("favorites");
    while (favDiv.childNodes.length > 0)
      favDiv.removeChild(favDiv.childNodes[0]);

    if (nFavorites < 1)
      {
        // If there are no favorites then display a helpful page instead
        var req = new XMLHttpRequest();
        req.open('GET',
                 'no-history-fallback.xhtml',
                 false);
        req.send(null);
        favDiv.innerHTML = "";
        favDiv.appendChild(req.responseXML.getElementById("no-history-root"));
      }
    else
      {
        var h1 = document.createElement('h1');
        h1.appendChild(document.createTextNode(_("Favorite pages")));
        favDiv.appendChild(h1);

        for (fav = 0; fav < nFavorites; fav++)
          {
            var div = document.createElement('div');
            div.setAttribute('class', 'favorite');

            // Create elements for the remove button. The outer div is
            // just used to make the position for the inner element
            // relative to the top of the favorite
            var removeOuter = document.createElement('div');
            removeOuter.setAttribute('class', 'remove-fav-outer');
            var removeInner = document.createElement('a');
            removeInner.setAttribute('class', 'remove-fav-inner');
            removeInner.setAttribute('href', 'javascript:');
            removeOuter.appendChild(removeInner);
            div.appendChild(removeOuter);

            // Create a closure with the url and use it as a callback
            // to remove the favorite when the link is clicked
            let (url = favorites[fav].url) {
              removeInner.addEventListener("click",
                                           function(e) {
                                             pagesService.removeFavorite(url);
                                             pagesService.getFavorites();
                                             e.stopPropagation();
                                           },
                                           false);
            }

            var a = document.createElement('a');
            div.appendChild(a);
            img = document.createElement('img');
            img.setAttribute('width', '223');
            img.setAttribute('height', '111');
            img.setAttribute('class', 'thumbnail');
            a.appendChild(img);
            a.appendChild(document.createElement('br'));
            var p = document.createElement('p');
            a.appendChild(p);

            p.appendChild(document.createTextNode(favorites[fav].title));
            a.setAttribute("href", favorites[fav].url);

            // Get a local URI for the thumbnail based on the page URL
            var thumbnailURI =
              pagesService.getThumbnailURL (favorites[fav].url);
            img.setAttribute("src", thumbnailURI.spec);

            favDiv.appendChild(div);
          }
      }
  },

  linkVisited : function(uri, visit_time, is_pinned)
  {
    if (is_pinned)
      rebuildPinnedPagesList();
  },

  pinnedPage : function(title, uri, visit_time)
  {
    rebuildPinnedPagesList();
  },

  unpinnedPage : function(uri)
  {
    rebuildPinnedPagesList();
  },

  QueryInterface : function(aIID)
  {
    if (!aIID.equals(MwbIPagesObserver)
        && !aIID.equals(nsISupports)
        && !aIID.equals(nsISupportsWeakReference))
      throw Components.results.NS_ERROR_NO_INTERFACE;
    return this;
  }
};

function rebuildPinnedPagesList()
{
  var div = document.getElementById('pinned_pages_div');
  var i;

  // Clear all existing entries
  while (div.childNodes.length > 0)
    div.removeChild(div.childNodes[0]);

  var nPages = pagesService.numPinnedPages;
  if (nPages > N_PINNED_PAGES)
    nPages = N_PINNED_PAGES;

  // If there are no pages then display some instructions instead
  if (nPages <= 0)
    div.innerHTML = ("<p>" + _("Push the pin icon in the location bar above to "
                               + "add pages to this list.") + "</p>");
  else
    {
      // Add each new entry
      for (i = 0; i < nPages; i++)
        {
          var page = pagesService.getPinnedPageByTime(i);
          var a = document.createElement('a');
          var pageDiv = document.createElement('div');
          var imgDiv = document.createElement('div');
          var img = document.createElement('img');
          var innerDiv = document.createElement('div');
          var titleSpan = document.createElement('span');

          a.setAttribute('href', page.uri);
          pageDiv.setAttribute('class', 'pinned_page_div');
          imgDiv.setAttribute('class', 'pinned_favicon_div');
          img.setAttribute('class', 'pinned_favicon');
          img.setAttribute('src', 'mwb-favicon://' + page.uri);
          imgDiv.appendChild(img);
          pageDiv.appendChild(imgDiv);
          innerDiv.setAttribute('class', 'pinned_inner_div');
          titleSpan.setAttribute('class', 'pinned_title');
          titleSpan.appendChild(document.createTextNode(page.title + " "));
          innerDiv.appendChild(titleSpan);
          if (page.visit_time > 0)
            {
              var visitTime = pagesService.formatTime(page.visit_time);
              var visitTimeSpan = document.createElement('span');
              visitTimeSpan.setAttribute('class', 'pinned_visit_time');
              visitTimeSpan.appendChild(document.createTextNode(visitTime));
              innerDiv.appendChild(visitTimeSpan);
            }
          pageDiv.appendChild(innerDiv);
          a.appendChild(pageDiv);
          div.appendChild(a);
        }

      if (pagesService.numPinnedPages >= 1)
        {
          var a = document.createElement('a');
          a.setAttribute('class', 'pinned_more_link');
          a.appendChild(document.createTextNode(_("More pinned pages")));
          a.setAttribute('href', 'pinnedpages.xhtml');
          var p = document.createElement('p');
          p.appendChild(a);
          div.appendChild(p);
        }
    }
}

function makeTabPrivate()
{
  pagesService.startPrivateBrowsing();
}

var observer;

// Run the initialisation once the page has finished loading
document.addEventListener("DOMContentLoaded", function(event)
{
  pagesService = (Components.classes["@moblin.org/mwb-pages-service;1"].
                  getService(Components.interfaces.MwbIPagesService));

  observer = new FetchFavsObserver();

  // Add our observer for when the favorites become available
  pagesService.addPagesObserver(observer);

  // Start fetching the favorites immediately
  pagesService.getFavorites();
  // and the pinned pages
  pagesService.getPinnedPages();
  rebuildPinnedPagesList();

  var privateButton = document.getElementById("make_private_button");
  privateButton.addEventListener("click", makeTabPrivate, false);

  // Set a style on the body tag if we are in private mode so that we
  // can style the page differently
  if (location.search.match(/^\?(?:|.+&)private(?:$|&)/))
    {
      var body = document.getElementsByTagName("body")[0];
      body.setAttribute("class", "private_mode");
    }

  // Unregister the load event so we don't initialize more than once
  event.currentTarget.removeEventListener("DOMContentLoaded",
                                          arguments.callee,
                                          false);
}, false);
